#include <sta/devices/arduino/bus/i2c.hpp>

#ifdef STA_PLATFORM_ARDUINO

#include <sta/devices/arduino/hal.hpp>
#include <sta/debug/debug.hpp>


namespace sta
{
#ifdef STA_PLATFORM_ARDUINO_DEVICE_ESP32
    ArduinoI2C::ArduinoI2C(Mutex * mutex /* = nullptr */, uint8_t sda /* = STA_ESP32_SDA_DEFAULT */, uint8_t scl /*= STA_ESP32_SCL_DEFAULT */, uint32_t frequency /* = 0U */)
        : I2C{mutex}
    {
        Wire.begin(sda, scl, frequency);
    }
#else
    ArduinoI2C::ArduinoI2C(Mutex * mutex /* = nullptr */)
        : I2C{mutex}
    {
        Wire.begin();
    }
#endif // STA_PLATFORM_ARDUINO_DEVICE_ESP32

    void ArduinoI2C::transfer(uint8_t value)
    {
        Wire.beginTransmission(address_);
        Wire.write(value);
        uint8_t code = Wire.endTransmission();

        if (code != 0)
            STA_DEBUG_PRINTF("Transfer result: %d", code); 
    }

    void ArduinoI2C::transfer16(uint16_t value)
    {
        Wire.beginTransmission(address_);
        Wire.write(value);
        uint8_t code = Wire.endTransmission();

        if (code != 0)
            STA_DEBUG_PRINTF("Transfer result: %d", code); 
    }

    void ArduinoI2C::transfer(const uint8_t * buffer, size_t size)
    {
        Wire.beginTransmission(address_);
        Wire.write(buffer, size);
        uint8_t code = Wire.endTransmission();

        if (code != 0)
            STA_DEBUG_PRINTF("Transfer result: %d", code);     
    }

    void ArduinoI2C::receive(uint8_t * buffer, size_t size)
    {
        size_t count = Wire.requestFrom((uint8_t)address_, (uint8_t)size);

        for (size_t i = 0; i < min(size, count); ++i)
        {
            buffer[i] = Wire.read();
        }
    }

    void ArduinoI2C::fill(uint8_t value, size_t count)
    {
        Wire.beginTransmission(address_);
        // Initialize a buffer of size count and fill it with the value.
        uint8_t buffer[count];
        memset(buffer, value, count);

        Wire.write(buffer, count);

        uint8_t code = Wire.endTransmission();

        if (code != 0)
            STA_DEBUG_PRINTF("Transfer result: %d", code);
    }

    ArduinoI2CDevice::ArduinoI2CDevice(I2C * intf, uint8_t address, bool master /* = true */, bool blocking /* = true */)
        : I2CDevice{intf, address, master, blocking}
    {

    }
} // namespace sta

#endif // STA_PLATFORM_ARDUINO